import errno, os
import ruamel.yaml as yaml
from getpass import getuser

default_user = getuser()

DEFAULTS = '''
    user: {user}
    explorer_path: /mnt/c/Windows/explorer.exe
    lxss_path: C:\\Users\\{user}\\AppData\\local\\lxss
    drives: [c, d]
'''.format(user=default_user)

class Config(yaml.comments.CommentedMap):
    def __init__(self, init):
        self.load(init)

    def dump(self):
        return yaml.dump(yaml.comments.CommentedMap(self),
                         Dumper=yaml.RoundTripDumper)

    def load(self, c):
        for key, value in yaml.load(c, Loader=yaml.RoundTripLoader).items():
            self[key] = value

    def save_file(self, filename):
        dirname = os.path.dirname(filename)
        if dirname:
            try:
                os.makedirs(dirname)
            except OSError as exception:
                if excpetion.errno != errno.EEXIST:
                    raise
        with open(filename, 'w') as f:
            f.write(self.dump())

    def load_file(self, filename):
        with open(filename) as f:
           self.load(f.read())



import click, os, subprocess
from re import match

import wslexplorer.config

@click.command()
@click.argument('path', type=click.Path(resolve_path=True, exists=True),
                default='.', required=False)
@click.option('config_file', '--config', '-c',
              type=click.Path(resolve_path=True, exists=True),
              help="Config file to use instead of default")
def launch(path, config_file=None):
    """Open Windows NT File Explorer at PATH from Windows Subsystem for Linux.  Default PATH is current directory."""
    config = wslexplorer.config.Config(wslexplorer.config.DEFAULTS)
    if config_file:
        config.load_file(config_file)
    else:
        config_path = os.path.join(click.get_app_dir('wsl-explorer'),
                                   'config.yaml')
        if os.path.exists(config_path):
            config.load_file(config_path)
        else:
            click.echo("No configuration found.")
            config['user'] = click.prompt('Windows username',
                                          config['user'])
            config.save_file(config_path)
            click.echo("Config saved to {}".format(config_path))

    #Convert POSIX path inside Windows WSL to a Windows path
    path = path.replace('/', '\\')
    reg_string = r'\\mnt\\([{}\z])'.format(''.join(config['drives']))
    m = match(reg_string, path)
    if m:
        prefix = '{}:'.format(m.group(1).upper())
        path = path[6:]
    else:
        prefix = config['lxss_path']
    
    with open(os.devnull, 'w') as FNULL:
        subprocess.call([config['explorer_path'], prefix + path],
                        stdout=FNULL, stderr=subprocess.STDOUT)



from setuptools import setup

with open ('README.rst') as f:
    long_description = f.read()

setup(
    name='wslexplorer',
    version='0.1.2',
    description='Simple script that launches Window Explorer from Windows Subsystem for Linux',
    long_description=long_description,
    url='https://github.com/scottfp/wslexplorer',
    author='Scott Pritchard',
    author_email='scottfp@gmail.com',
    license='MIT',
    py_modules=['wslexplorer.win_explorer',
                'wslexplorer.config'],
    install_requires=[
        'Click',
        'ruamel.yaml'
    ],
    entry_points='''
        [console_scripts]
        wslexplorer=wslexplorer.win_explorer:launch
    '''
)
